/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_filter.c,v 1.7 2004/02/18 03:08:12 ircd Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "hash.h"
#include "channel_mode.h"

#define FFILE DPATH "/etc/ircd.filter"

static void m_filter	(struct Client *, struct Client *, int, char **);

struct Message filter_msgtab = { "FILTER", 0, 0, 2, 2, MFLG_SLOW, 0, { m_ignore, m_filter, m_filter, m_filter } };

#ifndef STATIC_MODULE
void _modinit (void)
{
	mod_add_cmd (&filter_msgtab);
}

void _moddeinit (void)
{
	mod_del_cmd (&filter_msgtab);
}

const char *_version = "$Revision: 1.7 $";
#endif

static void m_filter	(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Channel *chptr;
	struct Filter *filter;
	FILE *fp;
	dlink_node *ptr, *tptr;
	char *tmp, *tmp2,  *ch, buf[BUFSIZE], word[BUFSIZE];
	int remove = 0, list = 0;

	ch = parv[1];
	tmp = ch;

	if ((parv[2] != NULL) && (!irccmp(parv[2], "LIST")) && (IsNetAdmin(source_p)))
		list = 1;

	if (parv[1][0] == '-')
	{
		tmp = strchr (ch, '-');
		*tmp = '\0';
		tmp++;
		remove = 1;
	}

	chptr = hash_find_channel (tmp);

	if (!chptr)
	{
		sendto_one (source_p, "%s :No such channel", tmp);
		return;
	}

	if (list == 1)
	{
		sendto_one (source_p, ":%s NOTICE %s :Filter list for %s", me.name, source_p->name, chptr->chname);
		DLINK_FOREACH (ptr, chptr->filterlist.head)
		{
			filter = ptr->data;
			sendto_one (source_p, ":%s NOTICE %s :Word %s", me.name, source_p->name, filter->word);
		}
		sendto_one (source_p, ":%s NOTICE %s :End of filter list", me.name, source_p->name);
		return;
	}

	if (!IsMember (source_p, chptr))
	{
		sendto_one(source_p, form_str(ERR_NOTONCHANNEL), me.name, source_p->name, tmp);
		return;
	}

	if (!is_chanop (source_p) && !IsNetAdmin(source_p))
	{
		sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED), me.name, source_p->name, tmp);
		return;
	}

	if (remove != 0)
	{
		sendto_one (source_p, ":%s NOTICE %s :Clearing filter for channel [%s]", me.name, source_p->name, chptr->chname);
		sendto_realops_flags (UMODE_SPY, L_ALL, "%s!%s@%s Clearing filter for channel [%s]", source_p->name, source_p->username, source_p->host, chptr->chname);
		sendto_server (NULL, NULL, CAP_ENCAP, NOCAPS, ":%s ENCAP * OPERSPY Clearing filter for channel [%s]", source_p->name, chptr->chname);

		DLINK_FOREACH_SAFE (ptr, tptr, chptr->filterlist.head)
		{
			filter = ptr->data;
			sendto_channel_local (ONLY_SERVERS, chptr, ":%s MODE %s -f %s", me.name, chptr->chname, filter->word);
			sendto_server (NULL, chptr, CAP_FILTER, NOCAPS, ":%s MODE %s -f %s", me.name, chptr->chname, filter->word);

			filter_del_word (chptr, filter->word);
		}

		chptr->filterlist.head = chptr->filterlist.tail = NULL;

		return;
	}

	if ((fp = fopen(ConfigFileEntry.filterfile, "r")) == NULL)
	{
		sendto_realops_flags (UMODE_DEBUG, L_ALL, "Failed opening %s: %s", ConfigFileEntry.filterfile, strerror(errno));
		return;
	}

	while (fgets (buf, BUFSIZE, fp))
	{
		if ((buf[0] != ';') && (buf[0] != '#') && (buf[0] != '\n') && (buf[0] != '\r'))
		{
			if (tmp2 = strchr (buf, '\n'))
				*tmp2 = '\0';
			if (tmp2 = strchr (buf, '\r'))
				*tmp2 = '\0';

			if (!buf)
				continue;

			if (filter_add_word (source_p, chptr, buf) == 1)
			{
				sendto_channel_local (ONLY_SERVERS, chptr, ":%s MODE %s +f %s", me.name, chptr->chname, buf);
				sendto_server (NULL, chptr, CAP_FILTER, NOCAPS, ":%s MODE %s +f %s", me.name, chptr->chname, buf);
			}
		}
	}

	fclose (fp);

	sendto_one (source_p, ":%s NOTICE %s :Populated channel filter [%s]", me.name, source_p->name, chptr->chname);
	sendto_realops_flags (UMODE_SPY, L_ALL, "%s!%s@%s Populating channel filter [%s]", source_p->name, source_p->username, source_p->host, chptr->chname);
	sendto_server (NULL, NULL, CAP_ENCAP, NOCAPS, ":%s ENCAP * OPERSPY Populating channel filter [%s]", source_p->name, chptr->chname);
	return;
}
