#include "stdinc.h"
#include "modules.h"
#include "hook.h"
#include "client.h"
#include "ircd.h"
#include "send.h"
#include "channel.h"
#include "channel_mode.h"
#include "numeric.h"
#include "hash.h"
#include "s_conf.h"
#include "hostmask.h" // find_address_conf()


int do_autojoin (struct hook_spy_data *);

#ifndef STATIC_MODULES
void
_modinit(void)
{
	hook_add_hook ("introduce_client", (hookfn *)do_autojoin);
}

void
_moddeinit(void)
{
	hook_del_hook ("introduce_client", (hookfn *)do_autojoin);
}

const char *_version = "$Revision: 1.2 $";
#endif

int do_autojoin (struct hook_spy_data *data)
{
	struct ConfItem* aconf;
	struct Channel *chptr;
	int type;
	char sjmode, *newch, mode, topic_info[USERHOST_REPLYLEN];
	struct Client *source_p;

	source_p = data->source_p;

	if (!MyClient(data->source_p))
		return 0;

	aconf = find_address_conf (source_p->rhost, source_p->username, &source_p->localClient->ip, source_p->localClient->aftype);

	if (!IsConfAutoJoin(aconf))
		return 0;

	if (AutoJoin.flags & AUTOJOIN_OP)
	{
		type = CHFL_CHANOP;
		mode = 'o';
		sjmode = '@';
	}
	else if (AutoJoin.flags & AUTOJOIN_HALFOP)
	{
		type = CHFL_CHANOP;
		mode = 'h';
		sjmode = '%';
	}
	else if (AutoJoin.flags & AUTOJOIN_VOICE)
	{
		type = CHFL_VOICE;
		mode = 'v';
		sjmode = '+';
	}
	else
	{
		type = CHFL_PEON;
		mode = sjmode = '\0';
	}

	newch = AutoJoin.channel;

	chptr = get_or_create_channel (data->source_p, newch, NULL);

	if (IsMember(data->source_p, chptr))
		return 0;

	if (is_banned(chptr, data->source_p) == CHFL_BAN)
	{
		sendto_one (data->source_p, form_str(ERR_BANNEDFROMCHAN), data->source_p->name, me.name, chptr->chname);
		return 0;
	}

    add_user_to_channel (chptr, data->source_p, type);
    sendto_server (data->source_p, chptr, NOCAPS, NOCAPS, ":%s SJOIN %lu %s + :%c%s", me.name, (unsigned long) chptr->channelts,
                    chptr->chname, type ? sjmode : ' ', data->source_p->name);
    sendto_channel_local (ALL_MEMBERS, chptr, ":%s!%s@%s JOIN :%s", data->source_p->name, data->source_p->username,
                    data->source_p->host, chptr->chname);

	if (type)
		sendto_channel_local (ALL_MEMBERS, chptr, ":%s MODE %s +%c %s", me.name, chptr->chname, mode, data->source_p->name);

	if (chptr->topic != NULL)
	{
		sendto_one (data->source_p, form_str(RPL_TOPIC), me.name, data->source_p->name, chptr->chname, chptr->topic);
		sendto_one (data->source_p, form_str(RPL_TOPICWHOTIME), me.name, data->source_p->name, chptr->chname, chptr->topic_info, chptr->topic_time);
	}
	else
	{
		ircsprintf (topic_info, "%s", me.name);
		set_channel_topic (chptr, AutoJoin.topic, topic_info, CurrentTime);
		sendto_server (NULL, chptr, NOCAPS, NOCAPS, ":%s TOPIC %s :%s", me.name, chptr->chname, (chptr->topic == NULL)? "" : chptr->topic);
		sendto_channel_local (ALL_MEMBERS, chptr, ":%s TOPIC %s :%s", me.name, chptr->chname, (chptr->topic == NULL)? "" : chptr->topic);
	}

	channel_member_names (chptr, data->source_p, 1);

	return 0;
}
