/*
 *  FreeWorld-ircd: an advanced Internet Relay Chat Daemon(ircd).
 *  m_global.c: Shows the current message of the day.
 *
 *  Copyright (C) 2004 FreeWorld-ircd Development Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_global.c,v 1.6.2.1 2004/04/02 19:51:45 jeremy Exp $
 */

#include "stdinc.h"
#include "client.h"
#include "tools.h"
#include "motd.h"
#include "ircd.h"
#include "send.h"
#include "numeric.h"
#include "handlers.h"
#include "hook.h"
#include "msg.h"
#include "s_serv.h"             /* hunt_server */
#include "parse.h"
#include "modules.h"
#include "s_conf.h"
#include "irc_string.h"

static void m_global (struct Client *, struct Client *, int, char **);
void ExpandParv(char *, int, int, int, char **);

struct Message global_msgtab = {
	"GLOBAL", 0, 0, 1, 0, MFLG_SLOW, 0,
	{m_unregistered, m_not_oper, m_ignore, m_global}
};

#ifndef STATIC_MODULES
void _modinit (void)
{
	mod_add_cmd (&global_msgtab);
}

void _moddeinit (void)
{
	mod_del_cmd (&global_msgtab);
}

const char *_version = "$Revision: 1.6.2.1 $";
#endif

/*
 * m_global
 *		parv[0] = sender prefix
 *		parv[1] = message
 */
static void m_global (struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Client *target_p;
	dlink_node *ptr;
	char msg[BUFSIZE], *type;
	int count = 0, startpoint = 1, umode = 0, local = 0;

	if (parc < 2)
	{
		sendto_one (source_p, "Syntax: [-all|-admins|-opers|-local] GLOBAL <message>");
		return;
	}

	type = NULL;

	if (!irccmp(&parv[1][0], "-admins"))
	{
		startpoint = 2;
		umode = UMODE_ADMIN;
		type = " [admins]";
	}
	else if (!irccmp(&parv[1][0], "-opers"))
	{
		startpoint = 2;
		umode = UMODE_OPER;
		type = " [opers]";
	}
	else if (!irccmp(&parv[1][0], "-local"))
	{
		startpoint = 2;
		local = 1;
		type = " [local]";
	}
	else if (!irccmp(&parv[1][0], "-all"))
		startpoint = 2;

	if (parv[startpoint] == '\0')
	{
		sendto_one (source_p, "Syntax: GLOBAL [-all|-admins|-opers] <message>");
		return;
	}

	msg[0] = '\0';
	ExpandParv (msg, BUFSIZE, startpoint, parc, parv);

	DLINK_FOREACH (ptr, global_client_list.head)
	{
		target_p = (struct Client *) ptr->data;
		if (IsPerson(target_p))
		{
			if (target_p == source_p)
				continue;

			if ((umode != 0) && !(target_p->umodes & umode))
				continue;

			if ((local != 0) && MyClient(target_p))
			{
			    sendto_one (target_p, ":%s!%s@%s NOTICE %s :{Global Notice%s} %s", source_p->name, source_p->username, source_p->host, target_p->name, type ? type : "", msg);
			    continue;
			}

			if (MyConnect(target_p))
				sendto_one (target_p, ":%s!%s@%s NOTICE %s :{Global Notice%s} %s", source_p->name, source_p->username, source_p->host, target_p->name,
							(type)? type : "", msg);
			else
				sendto_one (target_p, ":%s NOTICE %s :{Global Notice%s} %s", source_p->name, target_p->name, (type)? type : "", msg);
			count++;
		}
	}

	sendto_one (source_p, ":%s NOTICE %s :Global: (%s)", me.name, source_p->name, msg);
	sendto_one (source_p, ":%s NOTICE %s :sent to %d clients%s", me.name, source_p->name, count, (type)? type : "");
	sendto_realops_flags (UMODE_SPY, L_ALL, "%s sent GLOBAL", get_oper_name(source_p));
	sendto_server (NULL, NULL, CAP_ENCAP, NOCAPS, ":%s ENCAP * OPERSPY sent GLOBAL%s", source_p->name, (type)? type : "");

	return;
}

void ExpandParv(char *dest, int max, int initial, int parc, char *parv[])
{
  int i;

  if (parv[initial])
    strlcpy(dest, parv[initial], max);
  for (i = initial + 1; i < parc; i++)
  {
    strlcat(dest, " ", max);
    strlcat(dest, parv[i], max);
  }
}
