/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  FreeWorld-ircd:
 *  ipmask.c: IP Masking routines.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: ipmask.c,v 1.14 2004/02/23 23:00:32 jeremy Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "s_user.h"
#include "channel.h"
#include "channel_mode.h"
#include "class.h"
#include "client.h"
#include "common.h"
#include "fdlist.h"
#include "hash.h"
#include "irc_string.h"
#include "sprintf_irc.h"
#include "ircd.h"
#include "listener.h"
#include "motd.h"
#include "ircd_handler.h"
#include "msg.h"
#include "numeric.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "s_stats.h"
#include "scache.h"
#include "send.h"
#include "supported.h"
#include "whowas.h"
#include "md5.h"
#include "memory.h"
#include "packet.h"

void maskip (struct Client *);
char *ret_maskip (struct Client *);
int calculate_ip (struct Client *);

void maskip (struct Client *client_p)
{
	char buf[HOSTLEN];
	int i = 0,j = 0,k = 0;
	char *tmp = NULL;
	char *tmp2 = NULL;
	char *tmp3 = NULL;
	char *tmp4 = NULL;
	static char mask[HOSTLEN];

	strlcpy(buf, client_p->rhost, HOSTLEN);
	j = strlen(buf);
    mask[0] = '\0';

	if(strchr(buf, '.'))
	{
		k = calculate_ip (client_p);
		if(!isdigit(buf[j - 1]))
		{
			tmp = strchr(buf, '.');
			*tmp = '\0';
			tmp++;
			tmp2 = strchr(tmp, '.');

			if(tmp2 != NULL)
				ircsprintf(mask, "%s-%d.%s", ServerInfo.network_name, k, tmp);

		}
		else
		{
			tmp = strrchr(buf, '.');
			*tmp = '\0';
			tmp--;

			ircsprintf(mask, "%s.%d", buf, k);
		}

		if (mask[0] != '\0') {
			if (!strcasecmp(client_p->host, mask))
				return;
			sendto_realops_flags(UMODE_DEBUG, L_ALL, "Applying mask (%s) to %s", mask, client_p->name);
			strlcpy(client_p->host, mask, HOSTLEN);
		}
	}


#ifdef IPV6
	if(strchr(buf, ':'))
	{
		tmp = strrchr(buf, ':');
		*tmp = '\0';
		tmp--;
		tmp2 = strrchr(tmp, ':');

		if(tmp2 != NULL)
		{
			tmp3 = strrchr(tmp2, ':');

			if(tmp3 != NULL)
			{
				tmp4 = strrchr(tmp3, ':');
				*tmp4 = '\0';
				tmp4--;
			}
			ircsprintf(mask, "%s::IPV6", buf);
		}

		if (mask[0] != '\0') {
			if (!strcasecmp(client_p->host, mask))
				return;
			sendto_realops_flags(UMODE_DEBUG, L_ALL, "Applying mask (%s) to %s", mask, client_p->name);
			strlcpy(client_p->host, mask, HOSTLEN);
		}
	}
#endif

}

int calculate_ip (struct Client *client_p)
{
  char buffer[BUFSIZE];
  char *tmp, *tmp2, *tmp3;
  int a,b,c,d,e;

/*  if (!MyClient(client_p))
    return;*/

  strcpy(buffer, client_p->localClient->sockhost);

  if (strchr(buffer, ':'))
    return 0;

  if ((tmp = strchr(buffer, '.')) != NULL)
  {
    *tmp = '\0';
    tmp++;

    if ((tmp2 = strchr(tmp, '.')) != NULL)
    {
      *tmp2 = '\0';
      tmp2++;

      if ((tmp3 = strchr(tmp2, '.')) != NULL)
      {
        *tmp3 = '\0';
        tmp3++;
      }
    }
  }

  a = atoi(buffer);
  b = atoi(tmp);
  c = atoi(tmp2);
  d = atoi(tmp3);

  e = a + b + c + d;
  e = e * e * 1024 % 100000;

  return e;
}

char *ret_maskip(struct Client *client_p)
{
        char buf[HOSTLEN];
        int i = 0,j = 0,k = 0;
        char *tmp = NULL;
        char *tmp2 = NULL;
        char *tmp3 = NULL;
        char *tmp4 = NULL;
        static char mask[HOSTLEN];

        strlcpy(buf, client_p->rhost, HOSTLEN);
        j = strlen(buf);
        mask[0] = '\0';
  
        if(strchr(buf, '.'))
        {
                k = calculate_ip (client_p);
                if(!isdigit(buf[j - 1]))
                {
                        tmp = strchr(buf, '.');
                        *tmp = '\0';
                        tmp++;
                        tmp2 = strchr(tmp, '.');
  
                        if(tmp2 != NULL)
                                ircsprintf(mask, "%s-%d.%s", ServerInfo.network_name, k, tmp);
 
                }
                else
                {
                        tmp = strrchr(buf, '.');
                        *tmp = '\0';
                        tmp--;
  
                        ircsprintf(mask, "%s.%d", buf, k);
                }  

        }


#ifdef IPV6
        if(strchr(buf, ':'))
        {
                tmp = strrchr(buf, ':');
                *tmp = '\0';
                tmp--;
                tmp2 = strrchr(tmp, ':');

                if(tmp2 != NULL)
                {
                        tmp3 = strrchr(tmp2, ':');

                        if(tmp3 != NULL)
                        {
                                tmp4 = strrchr(tmp3, ':');
                                *tmp4 = '\0';
                                tmp4--;
                        }
                        ircsprintf(mask, "%s::IPV6", buf);
                }  

        }
#endif

	if (mask)
		return mask;
	else
		return client_p->host;
}
